from abc import ABC, abstractmethod

from azure.mgmt.compute.v2020_06_01 import ComputeManagementClient


class ArmClients(ABC):

    @abstractmethod
    def get_compute_client(self) -> ComputeManagementClient:
        pass



from .cli_client import ArmCliClients
from .interface import ArmClients
from .managed_identity_client import ArmManagedIdentityClients

__all__ = ['ArmClients', 'ArmManagedIdentityClients', 'ArmCliClients']



import logging
from typing import List

from azure.mgmt.compute.v2020_06_01.models import (VirtualMachineScaleSet,
                                                   VirtualMachineScaleSetVM)

from azure_helpers.clients import ArmClients
from azure_helpers.compute.vmss._create_helper import (VMSSProperties,
                                                       _VMSSCreateHelper)

log = logging.getLogger(__name__)


class VMSSHelper:
    """An encapsulation of actions required to manage a VMSS."""

    def __init__(self, arm_clients: ArmClients) -> None:
        self.arm_compute_client = arm_clients.get_compute_client()

    def create_or_update(self, vmss_properties: VMSSProperties) -> None:
        """Create a VMSS resource in azure.

        raises: `msrestazyre.azure_exceptions.CloudError` If there are any errors in the call to Azure.
        """
        log.info('Creating VMSS with name %s in azure.', vmss_properties.vmss_name)

        poller = self.arm_compute_client.virtual_machine_scale_sets.create_or_update(
            resource_group_name=vmss_properties.resource_group_name,
            vm_scale_set_name=vmss_properties.vmss_name,
            parameters=_VMSSCreateHelper(vmss_properties=vmss_properties).get_vmss_object()
        )
        log.info('Request sent to azure. Status: %s.', poller.status())
        poller.result()  # Wait synchronously
        log.info('Created VMSS with name %s successfully in azure.', vmss_properties.vmss_name)

    def get_vmss(self, resource_group_name: str, vmss_name: str) -> VirtualMachineScaleSet:
        """Get the VMSS based on resource group name and vmss name.

        raises: `msrestazyre.azure_exceptions.CloudError` If there are any errors in the call to Azure.
        """
        return self.arm_compute_client.virtual_machine_scale_sets.get(
            resource_group_name=resource_group_name,
            vm_scale_set_name=vmss_name,
        )

    def delete(self, resource_group_name: str, vmss_name: str) -> None:
        """Delete the VMSS.

        raises: `msrestazyre.azure_exceptions.CloudError` If there are any errors in the call to Azure.
        """
        log.info('Deleting VMSS with name %s in resource group %s.', vmss_name, resource_group_name)
        poller = self.arm_compute_client.virtual_machine_scale_sets.delete(
            resource_group_name=resource_group_name,
            vm_scale_set_name=vmss_name,
        )
        log.info('Delete request sent to azure. Status: %s.', poller.status())
        poller.result()  # Wait synchronously
        log.info('Successfully deleted VMSS with name %s in resource group %s.', vmss_name, resource_group_name)

    def get_all_vmss(self) -> List[VirtualMachineScaleSet]:
        """List all the VMSS for the compute client.

        raises: `msrestazyre.azure_exceptions.CloudError` If there are any errors in the call to Azure.
        """
        return list(self.arm_compute_client.virtual_machine_scale_sets.list_all())

    def get_all_vms_in_vmss(self, resource_group_name: str, vmss_name: str) -> VirtualMachineScaleSetVM:
        """List all the VMs in a VMSS.

        raises: `msrestazyre.azure_exceptions.CloudError` If there are any errors in the call to Azure.
        """
        return list(
            self.arm_compute_client.virtual_machine_scale_set_vms.list(
                resource_group_name=resource_group_name,
                virtual_machine_scale_set_name=vmss_name,
            )
        )



from msrestazure.azure_active_directory import MSIAuthentication

from .interface import ArmClients, ComputeManagementClient


class ArmManagedIdentityClients(ArmClients):

    def __init__(self, subscription_id: str) -> None:
        super().__init__()
        self.subscription_id = subscription_id

    def get_compute_client(self) -> ComputeManagementClient:
        return ComputeManagementClient(
            subscription_id=self.subscription_id,
            credentials=MSIAuthentication(),
        )



from dataclasses import dataclass
from typing import Dict, Optional

from azure.mgmt.compute.v2020_06_01.models import (
    ApiEntityReference, CachingTypes, ImageReference, LinuxConfiguration, Sku,
    UpgradeMode, UpgradePolicy, VirtualMachineScaleSet,
    VirtualMachineScaleSetExtension, VirtualMachineScaleSetExtensionProfile,
    VirtualMachineScaleSetIPConfiguration,
    VirtualMachineScaleSetNetworkConfiguration,
    VirtualMachineScaleSetNetworkProfile, VirtualMachineScaleSetOSDisk,
    VirtualMachineScaleSetOSProfile, VirtualMachineScaleSetStorageProfile,
    VirtualMachineScaleSetVMProfile)


@dataclass(init=True, eq=True, order=False)
class VMSSProperties:
    """A data-structure representing the properties of VMSS."""
    vmss_name: str
    resource_group_name: str
    location: str
    image_reference: ImageReference
    sku: Sku
    subnet_id: str
    admin_username: str
    admin_password: str
    tags: Dict[str, str]
    fault_domain_count: int = 1
    availability_zone: Optional[int] = 1


class _VMSSCreateHelper:
    """An encapsulation of actions required to create or update a VMSS object."""

    def __init__(self, vmss_properties: VMSSProperties) -> None:
        self.vmss_properties = vmss_properties

    def get_vmss_object(self) -> VirtualMachineScaleSet:
        """Get the azure VMSS object from the VMSSProperties."""
        return VirtualMachineScaleSet(
            location=self.vmss_properties.location,
            sku=self.vmss_properties.sku,
            upgrade_policy=UpgradePolicy(mode=UpgradeMode.automatic),
            virtual_machine_profile=VirtualMachineScaleSetVMProfile(
                os_profile=self._get_vm_os_profile(),
                network_profile=self._get_vm_network_profile(),
                storage_profile=self._get_storage_profile(),
                extension_profile=self._get_vm_extension_profile(),
            ),
            overprovision=True,
            do_not_run_extensions_on_overprovisioned_vms=True,
            tags=self.vmss_properties.tags,
            platform_fault_domain_count=self.vmss_properties.fault_domain_count,
            zones=[self.vmss_properties.availability_zone] if self.vmss_properties.availability_zone else None,
        )

    def _get_vm_os_profile(self) -> VirtualMachineScaleSetOSProfile:
        """Get the Virtual Machine OS profile for the VMSS."""
        return VirtualMachineScaleSetOSProfile(
            computer_name_prefix=self.vmss_properties.vmss_name,
            admin_username=self.vmss_properties.admin_username,
            admin_password=self.vmss_properties.admin_password,  # This is only good for testing. For production use case, consider encryption/SSH keys.
            linux_configuration=LinuxConfiguration(disable_password_authentication=False),
        )

    def _get_vm_network_profile(self) -> VirtualMachineScaleSetNetworkProfile:
        """Get the network profile associated with the VMSS."""
        return VirtualMachineScaleSetNetworkProfile(
                network_interface_configurations=[
                    VirtualMachineScaleSetNetworkConfiguration(
                        primary=True,
                        name='networkProfile',
                        ip_configurations=[
                            VirtualMachineScaleSetIPConfiguration(
                                primary=True,
                                name='IPConfiguration',
                                subnet=ApiEntityReference(id=self.vmss_properties.subnet_id),
                            )
                        ],
                    )
                ]
            )

    def _get_storage_profile(self) -> VirtualMachineScaleSetStorageProfile:
        """Get the storage profile associated with the VMSS."""
        return VirtualMachineScaleSetStorageProfile(
            image_reference=self.vmss_properties.image_reference,
            os_disk=VirtualMachineScaleSetOSDisk(create_option='fromImage', caching=CachingTypes.read_write),
            # data_disks=[VirtualMachineScaleSetDataDisk()]  # Add relevant data disks
        )

    def _get_vm_extension_profile(self) -> VirtualMachineScaleSetExtensionProfile:
        """Get the extensions for the VM."""
        return VirtualMachineScaleSetExtensionProfile(
            extensions=[
                VirtualMachineScaleSetExtension(
                    name='custom-script-runner',
                    publisher='Microsoft.Azure.Extensions',
                    type1='CustomScript',  # This azure SDK expects the type as `type1`. Don't ask me why.
                    type_handler_version='2.1',
                    settings={
                        'fileUris': [],  # You can download any scripts here. Maybe add to azure storage as well.
                        'commandToExecute': 'echo hello > /tmp/it_works.txt',
                    },
                ),
            ]
        )



from azure.common.client_factory import get_client_from_cli_profile

from .interface import ArmClients, ComputeManagementClient


class ArmCliClients(ArmClients):

    def __init__(self):
        super().__init__()

    def get_compute_client(self) -> ComputeManagementClient:
        return get_client_from_cli_profile(ComputeManagementClient)



import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="azure_helpers",
    version="0.0.2",
    author="Sanath Manavarte",
    author_email="msanath@gmail.com",
    description="A package which uses azure SDK for creating simple resources and managing them.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/msanath/azure-helpers.git",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'azure-cli-core',
        'azure-mgmt-compute==13.0.0'
    ]
)
